<?php

declare(strict_types=1);

namespace Creative\OAuth2\Client;

use League\OAuth2\Client\Provider\AbstractProvider;
use League\OAuth2\Client\Provider\Exception\IdentityProviderException;
use League\OAuth2\Client\Token\AccessToken;
use Psr\Http\Message\ResponseInterface;

/**
 * Class VkOAuthProvider.
 */
class VkOAuthProvider extends AbstractProvider
{
    protected const BASE_AUTH_URL = 'https://oauth.vk.com/authorize';
    protected const BASE_ACCESS_TOKEN_URL = 'https://oauth.vk.com/access_token';
    protected const BASE_API_URL = 'https://api.vk.com';

    /**
     * Api version.
     *
     * @var string
     *
     * @see https://vk.com/dev/versions
     */
    protected string $apiVersion;

    /**
     * VkOAuthProvider constructor.
     *
     * @param array $options
     * @param array $collaborators
     */
    public function __construct(array $options = [], array $collaborators = [])
    {
        parent::__construct($options, $collaborators);

        if (!isset($options['api_version'])) {
            throw new \InvalidArgumentException('The "api_version" option must be defined');
        }

        $this->apiVersion = (string) $options['api_version'];
    }

    /**
     * @inheritDoc
     *
     * @see https://vk.com/dev/authcode_flow_user
     */
    public function getBaseAuthorizationUrl(): string
    {
        return self::BASE_AUTH_URL;
    }

    /**
     * @inheritDoc
     *
     * @see https://vk.com/dev/authcode_flow_user
     */
    public function getBaseAccessTokenUrl(array $params): string
    {
        return self::BASE_ACCESS_TOKEN_URL;
    }

    /**
     * @inheritDoc
     *
     * @see https://vk.com/dev/users.get
     * @see https://vk.com/dev/objects/user
     * @see https://vk.com/dev/api_requests
     */
    public function getResourceOwnerDetailsUrl(AccessToken $token): string
    {
        $fields = [
            'id',
            'first_name',
            'last_name',
        ];

        $queryParamsString = $this->buildQueryString([
            'fields' => $fields,
            'access_token' => $token->getToken(),
            'v' => $this->apiVersion,
            'lang' => 'ru',
        ]);

        return self::BASE_API_URL . "/method/users.get?{$queryParamsString}";
    }

    /**
     * @inheritDoc
     *
     * @see https://vk.com/dev/permissions
     */
    protected function getDefaultScopes(): array
    {
        return [
            'email',
        ];
    }

    /**
     * @inheritDoc
     */
    protected function checkResponse(ResponseInterface $response, $data): void
    {
        if (isset($data['error']) || $response->getStatusCode() !== 200) {
            $errorMessage = $data['error_description'] ?? $data['error'] ?? $response->getReasonPhrase();
            throw new IdentityProviderException($errorMessage, $response->getStatusCode(), $data);
        }
    }

    /**
     * @inheritDoc
     */
    protected function createResourceOwner(array $response, AccessToken $token)
    {
        $data = $response['response'][0] ?? [];
        $data['email'] = $token->getValues()['email'] ?? null;

        return new VkResourceOwner($data);
    }
}
