<?php

declare(strict_types=1);

namespace Creative\OAuth2\Client\Tests;

use League\OAuth2\Client\Provider\Exception\IdentityProviderException;
use League\OAuth2\Client\Token\AccessToken;
use PHPUnit\Framework\TestCase;
use Psr\Http\Message\ResponseInterface;
use Creative\OAuth2\Client\VkOAuthProvider;
use Creative\OAuth2\Client\VkResourceOwner;

/**
 * Class VkOAuthProviderTest
 */
class VkOAuthProviderTest extends TestCase
{
    /**
     * @covers \Creative\OAuth2\Client\VkOAuthProvider
     */
    public function testConstructWithoutApiVersion(): void
    {
        $this->expectException(\InvalidArgumentException::class);

        new VkOAuthProvider([], []);
    }

    /**
     * @covers \Creative\OAuth2\Client\VkOAuthProvider
     */
    public function testGetResourceOwnerDetailsUrl(): void
    {
        $provider = new VkOAuthProvider(['api_version' => '1'], []);

        $token = '1212121';
        $accessToken = $this->createMock(AccessToken::class);
        $accessToken
            ->method('getToken')
            ->willReturn($token);

        $url = $provider->getResourceOwnerDetailsUrl($accessToken);
        $uri = \parse_url($url);
        \parse_str($uri['query'], $query);

        self::assertArrayHasKey('fields', $query);
        self::assertIsArray($query['fields']);

        self::assertContains('id', $query['fields']);
        self::assertContains('first_name', $query['fields']);
        self::assertContains('last_name', $query['fields']);

        self::assertArrayHasKey('access_token', $query);
        self::assertArrayHasKey('v', $query);
        self::assertArrayHasKey('lang', $query);
    }

    /**
     * @covers \Creative\OAuth2\Client\VkOAuthProvider
     */
    public function testCheckResponseWithError(): void
    {
        $reflection = new \ReflectionClass(VkOAuthProvider::class);
        $reflectionMethod = $reflection->getMethod('checkResponse');
        $reflectionMethod->setAccessible(true);

        $provider = new VkOAuthProvider(['api_version' => '1'], []);
        $response = $this->createMock(ResponseInterface::class);
        $response
            ->method('getStatusCode')
            ->willReturn(400);
        $response
            ->method('getReasonPhrase')
            ->willReturn('BAD_REQUEST');

        $this->expectException(IdentityProviderException::class);
        $reflectionMethod->invoke($provider, $response, []);
    }

    /**
     * @covers \Creative\OAuth2\Client\VkOAuthProvider
     * @covers \Creative\OAuth2\Client\VkResourceOwner
     */
    public function testCreateResourceOwner(): void
    {
        $reflection = new \ReflectionClass(VkOAuthProvider::class);
        $reflectionMethod = $reflection->getMethod('createResourceOwner');
        $reflectionMethod->setAccessible(true);

        $provider = new VkOAuthProvider(['api_version' => '1'], []);

        $email = 'mail@example.com';
        /** @var AccessToken $accessToken */
        $accessToken = $this->createMock(AccessToken::class);
        $accessToken
            ->method('getValues')
            ->willReturn([
                'email' => $email,
            ]);

        $response = [
            'id' => '1223',
            'first_name' => 'firstname',
            'last_name' => 'lastname',
        ];

        /** @var VkResourceOwner $resourceOwner */
        $resourceOwner = $reflectionMethod->invoke($provider, ['response' => [$response]], $accessToken);

        self::assertSame($response['id'], $resourceOwner->getId());
        self::assertSame($response['first_name'], $resourceOwner->getFirstName());
        self::assertSame($response['last_name'], $resourceOwner->getLastName());
        self::assertSame($email, $resourceOwner->getEmail());
    }
}
